/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <iostream.h> 
#include <qkeycode.h> 
#include <qstatusbar.h> 
#include <qlineedit.h> 
#include <qlabel.h> 
#include <qmultilineedit.h> 
#include <qpushbutton.h> 
#include "generate_sql_cache_entry.h" 
#include <qmessagebox.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qlayout.h>
#include <qpoint.h>
#include "dblist.h"
#include "widget.h"

#include "qt_oraconnect.h"
extern qt_oraconnect * gConnObj;

#include "generate.xpm" 
#include "execute.xpm" 
#include "tabledlg.h" 

//GenerateSQLCacheEntry::GenerateSQLCacheEntry(QWidget * parent, const char * name)
GenerateSQLCacheEntry::GenerateSQLCacheEntry(QWidget * parent, const QString& name)
       : Sheet(parent, name)
{
	setCaption("Adhoc SQL");

	// set the action menu
	fillActionMenu();

	Widget * centralWidget = new Widget(this);
	connect(centralWidget, SIGNAL(rightButtonPressed(const QPoint &)), this, SLOT(ContextMenu(const QPoint &))); 

	QBoxLayout *topLayout = new QVBoxLayout( centralWidget, 3 );
	QBoxLayout *table_select = new QHBoxLayout( topLayout);

	QLabel * l = new QLabel("Table or View: ",centralWidget);
	table_select->addWidget( l, 0 );

	table_field = new QLineEdit("",centralWidget);
	table_select->addWidget( table_field, 10 );
	connect(table_field,SIGNAL(returnPressed()),this,SLOT(generateEntry()));

	QPushButton * pb = new QPushButton("...",centralWidget);
	//pb->setGeometry(260,10,20,20);
	table_select->addWidget( pb, 0 );
	connect(pb,SIGNAL(clicked()),this,SLOT(showTableDlg()));

	l = new QLabel("'Select * from' query",centralWidget);
	topLayout->addWidget( l, 0 );

	select_field = new QMultiLineEdit(centralWidget);
	select_field->setWordWrap(QMultiLineEdit::WidgetWidth);
	topLayout->addWidget( select_field, 0 );

	centralWidget->setMinimumWidth(400);
	centralWidget->setMinimumHeight(200);
	setMinimumWidth(400);
	setMinimumHeight(200);
	setCentralWidget(centralWidget);
}

void GenerateSQLCacheEntry::fillActionMenu()
{
	getActionMenu()->insertSeparator();
        // set the action menu
        QPixmap generateIcon ;
        generateIcon = QPixmap( generate );
        getActionMenu()->insertItem(generateIcon,"Generate",this,SLOT(generateEntry()),CTRL+Key_G);
        QPixmap executeIcon ;
        executeIcon = QPixmap( exec );
        getActionMenu()->insertItem(executeIcon, "Execute SQL",this,SLOT(testSQL()),CTRL+Key_T);
	setupActionToolBar();

}

void GenerateSQLCacheEntry::setupActionToolBar()
{
	QToolBar* tb = getActionToolBar();
	tb->addSeparator();


        QPixmap generateIcon ;
        generateIcon = QPixmap( generate );
        QToolButton * generate = new QToolButton( generateIcon, "Generate", 0,
                                            this, SLOT(generateEntry()),
                                            tb, "Generate" );
        QPixmap executeIcon ;
        executeIcon = QPixmap( exec);
        QToolButton * execute = new QToolButton( executeIcon, "Execute", 0,
                                            this, SLOT(testSQL()),
                                            tb, "Execute" );
        generate->show();
}


GenerateSQLCacheEntry::~GenerateSQLCacheEntry()
{
#ifdef DESTRACE
	cerr << "GenerateSQLCacheEntry destructor" << endl;
#endif
}

void GenerateSQLCacheEntry::generateEntry()
{
    if (!gConnObj)
    {
		QMessageBox::critical( 0, "Database error", "You are not connected to a database!",QMessageBox::Ok | QMessageBox::Default,0,0);
		return;
	}
	try
	{
		QString select;
		gConnObj->open("select * from "+table_field->text()+" where rownum=0");
		int ncols=gConnObj->getCols();
		for (int i=0;i<ncols;i++)
		{
			if (i!=ncols-1)
				select=select+gConnObj->getColName(i)+", ";
			else
				select=select+gConnObj->getColName(i)+" ";
		}                                                                                                        
		gConnObj->close();
		select_field->setText("select "+select+"from "+table_field->text());
	}
	catch (oerr& e)
	{
		QMessageBox::critical( 0, "SQL error", QString("Oracle error: ") + ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
	}
}

void GenerateSQLCacheEntry::testSQL()
{
	DBList* w = new DBList(select_field->text(),0,"Testing 'select * from "+table_field->text()+"'",true);
	w->show();
}

void GenerateSQLCacheEntry::showTableDlg()
{
    TableDlg *td = new TableDlg(this);
    if (td->exec())
    {
	table_field->setText(td->getTable());
        delete td;
	td=0;
	generateEntry();
    }
    else
    {
        delete td;
	td=0;
    }
}
