/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "griddatadisplay.h" 
#include "row.h" 
#include <iostream.h> 
#include "qt_oraconnect.h"
#include <qheader.h>
#include <qmessagebox.h>
// For qt3
#include <qregexp.h>


GridDataDisplay::GridDataDisplay(qt_oraconnect * cObj, const QString& sql, 
				 QWidget * parent, const QString& name, bool ana)
	: ListView(parent,name), DataDisplay(cObj,sql), allowNumberAlign(ana)
{
	setAllColumnsShowFocus(TRUE);
}

long GridDataDisplay::retrieve(const ObjParm * parm)
{
   try
   {
	bool firstTime=false;
	if (!getLogMode()) clear();
	row* r;
	if (parm)
	{
                // Copy this parm to an qt_OraParm
                qt_OraParm op(parm->ParmCount());
                for (int i=0;i<parm->ParmCount();i++)
		{
                        op.addParm(parm->getParm(i));
		}
		getConnObj()->open(getSQL(),op);
	}
	else
		getConnObj()->open(getSQL());
	// Number of columns
	int ncols=getConnObj()->getCols();
	// only create the columns if they are not already there
	if (header()->count()==0)
	{
		// The fact that there are no columns probably means that this
		// is the first time that retrieve is called
		firstTime=true;		
		for (int i=0;i<ncols;i++)
		{
			addColumn(getConnObj()->getColName(i)); 
			// populate column list used for sort and filter dialogs
			addToColList(getConnObj()->getColName(i));
		}
	}
	// if this is not the initial query (at dd creation time)
	// and if a retrieve should be done at the initial query
	// then fetch the rows
	// else don't fetch the rows
	if (!(firstTime && getNoInitQuery()))
	{
		row *lastrow=0;
	        for (;getConnObj()->fetch() == 0;)
       		{
			r = new row(this,lastrow,allowNumberAlign);
			lastrow=r; // this pointer is needed to place the next row behind
			r->createCells(ncols);
       	         	// show the values of all the cols of this record
	                for (int i=0;i<ncols;i++)
       		        {
				QString colval = getConnObj()->getCol(i);
				// Remove \n from the data because the cause
				// Display problems in the list view
				r->setText(i,colval.replace(QRegExp("\n"),""));
       		        }
		}
	}
	long noRows=getConnObj()->count();
	getConnObj()->close();

	setSelected(firstChild(),true);
	return(noRows);
   }

   catch(oerr& e)
   {
	SQLErrorEvent(((qt_oerr&)e).getMsg());
	return -1;
   }
}

void GridDataDisplay::SQLErrorEvent(const QString& msg)
{
	emit SQLError(msg);
        // Unwind the waitCursors that may have been set elsewhere
        while (qApp->overrideCursor()) qApp->restoreOverrideCursor();
	QMessageBox::critical( 0, "SQL error", QString("Oracle error: ") + msg,QMessageBox::Ok | QMessageBox::Default,0,0);
}

void GridDataDisplay::print()
{
	printData();
}

void GridDataDisplay::save()
{
	saveData();
}
