/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#ifndef MDI_H
#define MDI_H

class QDomElement;
class QToolBar;
class QMenuBar;
class QPopupMenu;
class QDomNode;

#include "window.h"
#include "sheet.h"
#include <qintdict.h>
#include <qintcache.h>
#include <qvbox.h>
#include <qworkspace.h>

extern QString gCommandLineArg;
extern const QString gVersion;

class QMultiLineEdit;
class QToolBar;
class QPopupMenu;
class QWorkspace;

class mdi: public Window
{
    Q_OBJECT
public:
   	mdi();
   	virtual ~mdi();
   	uint registerSheet(const Sheet *s);
   	bool unregisterSheet(const uint index);
	bool deleteAllSheets();
	//void deleteAllSheets();
	void setActionMenu(QPopupMenu* m);
	QPopupMenu* getActionMenu();
	void setActionToolBar(QToolBar *tb);

	QWorkspace* getWorkspace();
	void replaceActionMenu(QPopupMenu * m);
	void replaceActionToolbar(QToolBar * tb);

	void removeActionItem();
	Sheet* getActiveSheet();

	const bool& getNoInitQuery() const;

	void addToVDollarMenu(QString name);

protected:
	virtual bool event ( QEvent * e );
	void commandLineLogon(const QString& connectString);
	void createToolBar();
	void createMenu();

	QString getOracleVersionString();
	void setupSQLCache();
	void setupMDITitle();
	bool connectToDB();
	void customizeSession();
	void registerApplication();

	void closeEvent( QCloseEvent* );

protected slots:

	void xmltest();
	void windowFactory(int menuid);
	QDomElement* findXmlWindow(const QString& w);
	
	// File menu
	void newDBAComp();
	bool logoff();
	//void logoff();
	void postInteractiveLogon();
	void interactive_logon();
	void quitDBAComp();

	// Database menu
	void database_overview();
	void dba_users();
	void dba_roles();
	void dba_profiles();
	void dba_tablespaces();
	void dba_rollback_segs();
	void dba_data_files();
	void v_log();
	void v_controlfile();
	void all_objects();
	void v_backup();
	void v_backup_v_datafile();
	void v_backup_corruption();
	void v_backup_datafile();
	void v_backup_device();
	void v_backup_piece();
	void v_backup_redolog();
	void v_backup_set();
	void v_recover_file();
	void v_recovery_log();
	void datafile_recovery_status();
	void v_recovery_file_status();
	void v_recovery_status();


	// Instance menu
	void instance_overview();
	void sga();
	void v_session();
	void sessions_processes();
	void sessions_transactions();
	void locks();
	void locktree();
	void audit_options();
	void object_audit_options();
	void default_audit_options();
	void audit_trail();
	void audit_sessions();
	void audit_objects();
	void mts();

	// performance menu
	void perf_indicators();
	void hot_files();
	void top_sql();

	// v$... menu
	void v_complete_list();
	void v_access();
	void v_archive();
	void v_archived_log();

	void v_bgprocess();
	void v_buffer_pool();

	void v_compatibility_compatseg();
	void v_controlfile_record_section();

	void v_datafile();
	void v_datafile_header();
	void v_dblink();
	void v_db_pipes();

	void v_event_name();

	void v_instance_recovery();

	void v_latch();
	void v_license();
	void v_lock();
	void v_locked_object();
	void v_log_history();

	void v_nls_database_parameters();
	void v_nls_instance_parameters();
	void v_nls_session_parameters();
	void v_nls_parameters();
	void v_nls_valid_values();

	void v_object_dependency();
	void v_obsolete_parameter();
	void v_option();

	void v_parameter();
	void v_pq_slave();
	void v_pq_sysstat();
	void v_process();
	void v_pwfile_users();

	void v_reserved_words();
	void v_resource_limit();
	void v_rollstat();
	void v_rowcache_parent();
	void v_rowcache_subordinate();

	void v_session_longops();
	void v_sesstat();
	void v_session_wait();
	void v_shared_pool_reserved();
	void v_sort_segment();
	void v_sort_usage();
	void v_sql_bind_data();
	void v_sql_bind_metadata();
	void v_sql_cursor();
	void v_sql_shared_memory();
	void v_subcache();
	void v_sysstat();
	void v_system_event();

	void v_tablespace();
	void v_tempfile();
	void v_transaction();

	void v_waitstat();

	// Reports
	void tbs_sizes();
	void extents_left();
	void free_space();
	void free_space_deficit();
	void invalid_objects();

	// Tools menu
	void clearSQLCache();
	void UserDefinedSQL();
	void SQLPlus();
	void SQLEditor();
	void generateSQLCacheEntry();
	void generate_crdbs_config();
	void generate_database_create();
	void toggleNoInitQuery();

	// Help menu
	void aboutDBAComp();
	void aboutQt();

	void callVDollar(int name);

    
private slots:
	void SheetActivated(QWidget *w);
	void fillWindowsMenu();
	void windowsMenuActivated(int id);

private:
    	QIntDict<Sheet> *SheetList;
    	uint SheetSeqNo;

	QWorkspace *ws;
	QToolBar* ToolBar;
	QPopupMenu* action;
	int ActionMenuID;
	QToolBar* actionToolBar;
	QPopupMenu* windows;
	int FileMenuID;

	void changeMenuState(bool enabled);
	virtual void generate_sql(const QString& fn, const QString& what);

	Sheet * activeSheet;

	void build_menus(QDomNode n, QMenuData* menu);

	bool noInitQuery;
	int NIQItem; // no initial query mode menu item
	QPopupMenu * tools; // pointer to the tools submenu

	QString mdiBaseTitle; // title of the mdi window

	QPopupMenu * vdollar;
	QIntDict<QString> *vdollarMenuCache;

	void buildVDollarMenu();
	void saveVDollarMenu();

};


#endif
