/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "multilineedit.h" 
#include "row.h"
#include "finddlg.h"
#include "replacedlg.h"
//#include <qprinter.h>
#include <iostream.h>
#include <qpaintdevicemetrics.h>
#include <qheader.h>
#include <qfiledialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>
// for qt3
#include <qpainter.h>
#include <qglobal.h>
#if QT_VERSION > 0x020302
#define QT3
#endif


//MultiLineEdit::MultiLineEdit(QWidget * parent, const char * name) 
MultiLineEdit::MultiLineEdit(QWidget * parent, const QString& name) 
	:QMultiLineEdit(parent,name.local8Bit())
{
}

void MultiLineEdit::printData(const QString& OutputFilename, const QString& docName, const QPrinter::Orientation& orient)
{
    const int Margin = 10;
    int pageNo = 1;
    QPrinter *printer = new QPrinter();

    if ( printer->setup(this) ) {               // printer dialog
        //statusBar()->message( "Printing..." );
        QPainter p;
        if( !p.begin( printer ) )
            return;                             // paint on printer

        p.setFont( font() );
        int yPos        = 0;                    // y position for each line
        QFontMetrics fm = p.fontMetrics();
        QPaintDeviceMetrics metrics( printer ); // need width/height
                                                // of printer surface
        for( int i = 0 ; i < numLines() ; i++ ) {
            if ( Margin + yPos > metrics.height() - Margin ) {
                QString msg( "Printing (page " );
                msg += QString::number( ++pageNo );
                msg += ")...";
                //statusBar()->message( msg );
                printer->newPage();             // no more room on this page
                yPos = 0;                       // back to top of page
            }
            p.drawText( Margin, Margin + yPos,
                        metrics.width(), fm.lineSpacing(),
                        ExpandTabs | DontClip,
                        textLine( i ) );
            yPos = yPos + fm.lineSpacing();
        }
        p.end();                                // send job to printer
        //statusBar()->message( "Printing completed", 2000 );
    } else {
        //statusBar()->message( "Printing aborted", 2000 );
    }
}

void MultiLineEdit::saveData()
{
        QString filename=QFileDialog::getSaveFileName("", "*.txt", this );
        if ( !filename.isNull() )
        {
                QFile f(filename);
                if (f.open(IO_WriteOnly))
                {
                        QTextStream ts(&f);
			ts<<text();
			f.close();
                }
        }
}

void MultiLineEdit::findWord(const QString& word,bool down, 
			     int startRow, int startCol)
{
	QString line;
	int wordPosition;
	int wordLength=word.length();
	if (startRow==-1 || startCol==-1) 
	{
		getCursorPosition(&startRow,&startCol);
	}
		
        for (int row=startRow;((row>=0) && (row<numLines()));)
        {
		line = textLine(row);
		// Before
		//line = *getString(row);
		// for the first row checked startCol
		// maybe 0 or >0 at this point

		if (down) wordPosition=line.find(word,startCol,false);
		//else wordPosition=line.findRev(word,startCol,false);
		else 
		{
			if (startCol==-1) startCol=line.length();
			wordPosition=line.findRev(word,startCol-line.length()-1,false);
		}

		// for subsequent rows it must be 0 or -1 again
		if (down) startCol=0;
		else startCol=-1;
		if (wordPosition!=-1)  // found the word
		{
/* setCursor does not behave the same in Qt3 as in QT2 so I'm now using setSelection
** which is not implemented in QT2
*/
#ifdef QT3
                        if (down)
                        {
                                setSelection(row,wordPosition,row,wordPosition+wordLength);
                        }
                        else
                        {
                                setSelection(row,wordPosition+wordLength,row,wordPosition);
                        }
#else
			//setSelection(row,wordPosition,row,wordLength);
			// set Selection is not implemented in Qt 2.3.2
			// here is the woraround
			if (down)
			{
				setCursorPosition (row, wordPosition, FALSE);
				setCursorPosition (row, wordPosition+wordLength, TRUE);
			}
			else
			{
				setCursorPosition (row, wordPosition+wordLength, FALSE);
				setCursorPosition (row, wordPosition, TRUE);
			}
#endif

			return;
		}
		if (down) row++;
		else row--;

		// Check wrap around conditions
		if (row==-1) // Reached top of document
		{
			if (QMessageBox::information(this, "Find", "Reached beginning of document do you want to continue from the end?",QMessageBox::Yes | QMessageBox::Default,QMessageBox::No,0)==QMessageBox::Yes) row=numLines()-1;
		}
		else 
		{
			if (row==numLines()) // Reached end of document
				if (QMessageBox::information(this, "Find", "Reached end of document do you want to continue from the beginning?",QMessageBox::Yes | QMessageBox::Default,QMessageBox::No,0)==QMessageBox::Yes) row=0;
		}
			
        }
}

void MultiLineEdit::findDialog()
{
	FindDlg *fd = new FindDlg(QString::null, this);
	while (fd->exec())
        {
		findWord(fd->getWord(), fd->getUp());
	}
	delete fd;
	fd=0;
}

void MultiLineEdit::replaceWord(const QString& newWord)
{
	backspace();
	insert(newWord,false);
}

void MultiLineEdit::replaceDialog()
{
	ReplaceDlg *fd = new ReplaceDlg(QString::null, this);
	while (fd->exec())
        {
		if (fd->getAction()=="FIND")
				findWord(fd->getWord(), fd->getUp());
		else if (fd->getAction()=="REPLACE")
		{

			if (markedText().lower()!=fd->getWord().lower())
				findWord(fd->getWord(), fd->getUp());
			else 
			{
				replaceWord(fd->getNewWord());
				findWord(fd->getWord(), fd->getUp());
			}
		}
		else if (fd->getAction()=="REPLACE ALL")
		{
			
			if (markedText().lower()!=fd->getWord().lower())
				findWord(fd->getWord(), fd->getUp());
			while (markedText().lower()==fd->getWord().lower())
			{
				replaceWord(fd->getNewWord());
				findWord(fd->getWord(), fd->getUp());
			}
		}

	}
	delete fd;
	fd=0;
}

