/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#ifndef TABDATADISPLAY_H
#define TABDATADISPLAY_H  

#include "tabview.h"
#include "datadisplay.h"
#include <qdict.h>
class GridDataDisplay;
class FormDataDisplay;
class MLEDataDisplay;

class TabDataDisplay: public TabView, public DataDisplay
{
	Q_OBJECT
public:
	TabDataDisplay(qt_oraconnect * cObj=0, QWidget * parent=0, 
		       const QString& name=QString::null);
	//TabDataDisplay(qt_oraconnect * cObj=0, QWidget * parent=0, const char * name=0);
	virtual ~TabDataDisplay();

	virtual void addTab(QWidget *widget, const QString& name);

	virtual void addGridTab(GridDataDisplay *grid, const QString& name);
	virtual void addGridTab(QString sql, const QString& name, bool ana=true);

	virtual void addFormTab(FormDataDisplay *form, const QString& name);
	virtual void addFormTab(QString sql, const QString& name, int cols=1, 
				int lw=100, int tw=100, 
				QString SingleField=QString::null,int sfw=0, 
				int sfh=0);

	virtual void addMLETab(MLEDataDisplay *mle, const QString& name);
	virtual void addMLETab(QString sql, const QString& name,
			       bool headers=false, QString line_sep="\n", 
			       QString col_sep="\t", QString head_sep="\n");

	virtual void addTabTab(TabDataDisplay *tab, const QString& name);

	virtual long retrieve(const ObjParm * parm=0);
	virtual QPopupMenu* getMenuAddition();

	virtual void setSort(const QString& s);
	virtual void setFilter(const QString& f);
	virtual const QStringList& getColList() const;
    	virtual const QString& getSort() const;
    	virtual const QString& getFilter() const;                                   
    	virtual const QString& getSQL(); //const;
    	//virtual const char* getSQL(); //const;

	virtual const QString getCurrentTab();

	virtual void print();
	virtual void save();
	virtual const bool getLogMode() const;
	virtual void setLogMode(const bool ror);

//	virtual const bool& getNoInitQuery() const;
//	virtual void setNoInitQuery(const bool niq);

signals:
	void SQLError(const QString& msg);
	void subTabSelected ( const QString & SubTabName); 

public slots:
	
private:
	struct TabContent
	{
		bool isDD;
		QWidget* widget;
		DataDisplay* dd;
	};

	QDict<TabContent> TabContents;
	QString currentTab;

private slots:
	void setCurrentTab(const QString & name );
	void translateSubtabNameToTabName(const QString & SubTabName);
	/* This method is meant to be connected to the signal subTabSelected
           It translate the sub tab name to the ancestor tab name and signals
	   'selected' to the ancestor tab and that will eventually activate
	   the refresh of the sub tab */

};


#endif  
