/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "tabledlg.h"
#include <qpushbutton.h>
#include <qlistbox.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include "sqlcache.h"
#include "qt_oraconnect.h"

extern qt_oraconnect * gConnObj;
extern SQLCache * gSQLCache;


TableDlg::TableDlg(QWidget *parent, QString name )
        : QDialog( parent, name.local8Bit(), TRUE )
{
	QPushButton *ok, *cancel;
	ok = new QPushButton( "OK", this );
	ok->setGeometry( 300,10, 90,30 );
	connect( ok, SIGNAL(clicked()), SLOT(accept()) );
	cancel = new QPushButton( "Cancel", this );
	cancel->setGeometry( 300,60, 90,30 );
	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );

	QLabel *l=new QLabel("Tables and Views",this);
	l->setGeometry(10,5,100,20);
	TableList = new QListBox(this);
	TableList->setGeometry(10,25,250,260);
	connect( TableList, SIGNAL(doubleClicked( QListBoxItem *)), SLOT(accept()) );
	setFixedWidth(400);
	setFixedHeight(300);                                                        

	ok->setFocus();
	setCaption("Select table or view");

    if (!gSQLCache)
    {
		QMessageBox::critical( 0, "SQL cache error", QString("The SQL cache is not set up"),QMessageBox::Ok | QMessageBox::Default,0,0);
        return;                                                          
	}

    if (!gConnObj)
    {
        QMessageBox::critical( 0, "Database error", "You are not connected to a database!",QMessageBox::Ok | QMessageBox::Default,0,0);
        return;                                                          
	}

	// Fill list box
	try
	{
		gConnObj->open(gSQLCache->getSQL("all_tables"));
		for (;gConnObj->fetch() == 0;)
		{
			TableList->insertItem(gConnObj->getCol(0));
		}
		gConnObj->close();
	}
	catch(oerr& e)
	{
		QMessageBox::critical( 0, "SQL error", QString("Oracle error: ") + ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
	}
}

TableDlg::~TableDlg()
{
}

const QString TableDlg::getTable() const
{
	return TableList->currentText();
}

QListBox* TableDlg::getTableList() const
{
	return TableList;
}
