/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <iostream.h>
#include <qapplication.h>
#include "w_dba_indexes.h"
#include "w_dba_indexes_info.h"

#include <stdlib.h>
#include <qmessagebox.h>
#include "scriptdlg.h"
#include "gensql.h"
#include "editor.h"
#include "config.h"
#include <qfile.h>
#include <qtextstream.h>


w_dba_indexes::w_dba_indexes()
	: DBList("dba_indexes",0,"DBA_INDEXES")
{
	addGenerateSQLToActionMenu();
	show();
}

w_dba_indexes::w_dba_indexes(const ObjParm& parm)
	: DBList(parm,"dba_indexes_per_user",0,"DBA_INDEXES for user "+parm.getParm(0))
{
        //setCaption("DBA_INDEXES for user "+parm.getParm(0));
	addGenerateSQLToActionMenu();
	show();
}

QPopupMenu * w_dba_indexes::addGenerateSQLToActionMenu()
{
        QPopupMenu * m= DBList::addGenerateSQLToActionMenu();
        m->insertItem("Create...",this,SLOT(generate_index_create()));
        m->insertItem("Drop...",this,SLOT(generate_index_drop()));
        m->insertItem("Rebuild...",this,SLOT(generate_index_rebuild()));
        return m;
}


void w_dba_indexes::callInfo(row * clickedRow)
{
	ObjParm parm;
	if (getWinParm())
	{ // We are in the per_user case
		parm.addParm(getWinParm()->getParm(0)); // username
	}
	else
	{ // We are in the all objects case
		parm.addParm(clickedRow->getCellbyName("owner"));
	}
	parm.addParm(clickedRow->getCellbyName("index_name"));
	w_dba_indexes_info *w = new w_dba_indexes_info(parm);
}

void w_dba_indexes::generate_index_drop()
{
        generate_sql("index_drop.sql","index_drop");
}

void w_dba_indexes::generate_index_create()
{
        generate_sql("index_create.sql","index_create");
}

void w_dba_indexes::generate_index_rebuild()
{
        generate_sql("index_rebuild.sql","index_rebuild");
}

void w_dba_indexes::generate_sql(const QString& fn, const QString& what)
{
        QString owner,filename,object_name,table_name;
        bool    allobj, show_script, echo_on, spool;

        gConfigFile->setGroup("PATH");
        QString path = gConfigFile->readEntry("sql_generation",".");


        ScriptDlg *sd = new ScriptDlg(path+"/"+fn,this);
        if (sd->exec())
        {
		qApp->setOverrideCursor(waitCursor);
                filename=sd->getFilename();
                allobj=sd->getAllObj();
                show_script=sd->getShow();
                spool=sd->getSpool();
                echo_on=sd->getEchoOn();
                delete sd;
		sd=0;

                QFile f(filename);
                if (f.open(IO_WriteOnly))
                {
                        QTextStream t(&f);
                        if (spool) t << "spool " << filename << ".log" << endl;
                        if (echo_on) t << "set echo on" << endl;
                        if (!allobj) // generate script for selected object
                        {
                                object_name=((row *)(((GridDataDisplay *)getDD())->currentItem()))->getCellbyName("index_name");
                                if (getWinParm())
                                { // We are in the per_user case
                                        owner=getWinParm()->getParm(0);
                                }
                                else
                                { // We are in the all objects case
                                        owner=((row *)(((GridDataDisplay *)getDD())->currentItem()))->getCellbyName("owner");
                                }
                                int retval=0;
                                if (what=="index_create")
                                        retval=GenSQL::IndexCreate(t,owner,object_name);
                                else if (what=="index_rebuild")
                                        retval=GenSQL::IndexRebuild(t,owner,object_name);
                                else if (what=="index_drop")
                                        retval=GenSQL::IndexDrop(t,owner,object_name);
                                if (retval==-1)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "File write error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                                else if (retval==-2)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "Database error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                                else if (retval==-3)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "SQL cache error. SQL could not be found!\nProcessing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                        }
                        else // generate script for all objects
                        {
                                QListViewItemIterator it((GridDataDisplay *)getDD());
                                // loop over all rows
                                for (;it.current(); ++it)
                                {
                                        object_name=((row *)it.current())->getCellbyName("index_name");
                                        if (getWinParm())
                                        { // We are in the per_user case
                                                owner=getWinParm()->getParm(0);
                                        }
                                        else
                                        { // We are in the all objects case
                                                owner=((row *)it.current())->getCellbyName("owner");
                                        }
                                        int retval=0;
                                        if (what=="index_create")
                                                retval=GenSQL::IndexCreate(t,owner,object_name);
                                        if (what=="index_drop")
                                                retval=GenSQL::IndexDrop(t,owner,object_name);
                                        if (what=="index_rebuild")
                                                retval=GenSQL::IndexRebuild(t,owner,object_name);
                                        if (retval==-1)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "File write error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                        else if (retval==-2)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "Database error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                        else if (retval==-3)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "SQL cache error. SQL could not be found!\nProcessing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                }
                        }
                        if (echo_on) t << "set echo off" << endl;
                        if (spool) t << "spool off" << endl;
                        f.close();
                }

                if (show_script)
                {
                        Editor * ed = new Editor(filename);
                        ed->show();
                }
		qApp->restoreOverrideCursor();
        }
        else
        {
                delete sd;
		sd=0;
        }
}

