/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <iostream.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include "row.h"
#include "dbtab.h"
#include "dbform.h"
#include "griddatadisplay.h"
#include "formdatadisplay.h"
#include "w_dba_tablespaces_info.h"
#include "dblist.h"
#include "objparm.h"
#include "qt_oraconnect.h"
#include "sqlcache.h"
#include "scrollview.h"
#include <qobjcoll.h>
#include <qtoolbutton.h>

w_dba_tablespaces_info::w_dba_tablespaces_info(const ObjParm& parm)
	: DBTab(parm,0,"Information for tablespace "+parm.getParm(0)), 
	  fspace_alloc(0), flargest_alloc(0), fsmallest_alloc(0),
	  fextents_alloc(0), fspace_free(0), flargest_free(0), 
	  fsmallest_free(0), fextents_free(0), fsize(0)
{
	//setCaption("Information for tablespace "+parm.getParm(0));
	makeTabs();
        show();
	// from now on changing to the space tab should fill it
	// This connect must be here. If you put it earlier, the fill code 
	// will be executed already during window creation making this much slower
// NO LONGER NECESSARY AS RETRIEVE IS REIMPLEMENTED
//	connect((TabDataDisplay*)getDD(), SIGNAL(selected(const QString &)), this, SLOT(fillSpaceTab(const QString & )));
}

void w_dba_tablespaces_info::makeTabs()
{
	addGridTab("dba_data_files_per_tbs","Datafiles");
	addGridTab("dba_segments_per_tbs","Segments");
	addGridTab("dba_users_def_per_tbs","Def users");
	addGridTab("dba_users_temp_per_tbs","Temp users");
	makeSpaceTab();
	addGridTab("v_access_per_tbs","Access");
	addGridTab("tbs_map","Map");
	addGridTab("dba_free_space_per_tbs","Free");
	addFormTab("v_sort_segment_per_tbs","Sort segment",2,130,130);
	addGridTab("v_sort_usage_per_tbs","Sort usage");
}

void w_dba_tablespaces_info::makeSpaceTab()
{
	const int startx=10;
	const int starty=50;
	const int lw=125;
	const int lh=20;
	const int fw=90;
	const int fh=20;
	const int hor_dist=10;
	const int ver_dist=20;
	const int col_dist=30;
	const int startx_col2=startx+lw+hor_dist+fw+col_dist;



	// Make it now so that it appears in the tab bar
	//SpaceTab=new QWidget;
	SpaceTab=new ScrollView(this);
	SpaceTab->resizeContents(530,289);
	SpaceTab->setVScrollBarMode(QScrollView::Auto);
	SpaceTab->setHScrollBarMode(QScrollView::Auto);
	SpaceTab->enableClipper(TRUE);

	SpaceTab->viewport()->setBackgroundMode(PaletteBase);

    	connect(SpaceTab, SIGNAL(shown()), this, SLOT(SpaceTabShown()));
    	connect(SpaceTab, SIGNAL(hidden()), this, SLOT(SpaceTabHidden()));
    	connect(SpaceTab, SIGNAL(rightButtonPressed(const QPoint &)), this, 
		SLOT(ContextMenu(const QPoint &)));       

	QLabel * title=new QLabel("Space allocated",SpaceTab->viewport());
        title->resize(150,20);
	title->setBackgroundMode(PaletteBase);
        SpaceTab->addChild(title,60,20);
	QFont curr_font=title->font();
	curr_font.setBold(true);
	curr_font.setUnderline(true);
	title->setFont(curr_font);

        // Extents allocated
                QLabel * lextents_alloc = new QLabel("Extents:",SpaceTab->viewport());
		lextents_alloc->resize(lw,lh);
		lextents_alloc->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(lextents_alloc,startx,starty+0*(lh+ver_dist));

                fextents_alloc = new QLabel("",SpaceTab->viewport());
                fextents_alloc->resize(fw,fh);
		fextents_alloc->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(fextents_alloc,startx+lw+hor_dist,starty+0*(fh+ver_dist));
                fextents_alloc->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
                fextents_alloc->setFrameStyle(QFrame::Box|QFrame::Plain);

                // Smallest extent allocated
                QLabel * lsmallest_alloc = new QLabel("Smallest:",SpaceTab->viewport());
		lsmallest_alloc->resize(lw,lh);
		lsmallest_alloc->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(lsmallest_alloc,startx,starty+1*(lh+ver_dist));

                fsmallest_alloc = new QLabel("",SpaceTab->viewport());
                fsmallest_alloc->resize(fw,fh);
		fsmallest_alloc->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(fsmallest_alloc,startx+lw+hor_dist,starty+1*(lh+ver_dist));
                fsmallest_alloc->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
                fsmallest_alloc->setFrameStyle(QFrame::Box|QFrame::Plain);

                // Largest extent allocated
                QLabel * llargest_alloc = new QLabel("Largest:",SpaceTab->viewport());
                llargest_alloc->resize(lw,lh);
		llargest_alloc->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(llargest_alloc,startx,starty+2*(lh+ver_dist));
                flargest_alloc = new QLabel("",SpaceTab->viewport());
                flargest_alloc->resize(fw,fh);
		flargest_alloc->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(flargest_alloc,startx+lw+hor_dist,starty+2*(fh+ver_dist));
                flargest_alloc->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
                flargest_alloc->setFrameStyle(QFrame::Box|QFrame::Plain);

                // Space allocated
                QLabel * lspace_alloc = new QLabel("Space allocated (MB):",SpaceTab->viewport());
                lspace_alloc->resize(lw,lh);
		lspace_alloc->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(lspace_alloc,startx,starty+3*(lh+ver_dist));
                fspace_alloc = new QLabel("",SpaceTab->viewport());
                fspace_alloc->resize(fw,fh);
		fspace_alloc->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(fspace_alloc,startx+lw+hor_dist,starty+3*(fh+ver_dist));
                fspace_alloc->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
                fspace_alloc->setFrameStyle(QFrame::Box|QFrame::Plain);

        title=new QLabel("Free space",SpaceTab->viewport());
        title->resize(150,20);
	title->setBackgroundMode(PaletteBase);
        SpaceTab->addChild(title,350,20);
        curr_font=title->font();
        curr_font.setBold(true);
        curr_font.setUnderline(true);
        title->setFont(curr_font);


        // Extents free
                QLabel * lextents_free = new QLabel("Extents:",SpaceTab->viewport());
                lextents_free->resize(lw,lh);
		lextents_free->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(lextents_free,startx_col2,starty+0*(lh+ver_dist));
                fextents_free = new QLabel("",SpaceTab->viewport());
                fextents_free->resize(fw,fh);
		fextents_free->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(fextents_free,startx_col2+lw+hor_dist,starty+0*(fh+ver_dist));
                fextents_free->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
                fextents_free->setFrameStyle(QFrame::Box|QFrame::Plain);

                // Smallest extent free
                QLabel * lsmallest_free = new QLabel("Smallest:",SpaceTab->viewport());
                lsmallest_free->resize(lw,lh);                
		lsmallest_free->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(lsmallest_free,startx_col2,starty+1*(lh+ver_dist));
                fsmallest_free = new QLabel("",SpaceTab->viewport());
                fsmallest_free->resize(fw,fh);
		fsmallest_free->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(fsmallest_free,startx_col2+lw+hor_dist,starty+1*(fh+ver_dist));
                fsmallest_free->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
                fsmallest_free->setFrameStyle(QFrame::Box|QFrame::Plain);

                // Largest extent free
                QLabel * llargest_free = new QLabel("Largest:",SpaceTab->viewport());
                llargest_free->resize(lw,lh);                
		llargest_free->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(llargest_free,startx_col2,starty+2*(lh+ver_dist));
                flargest_free = new QLabel("",SpaceTab->viewport());
                flargest_free->resize(fw,fh);
		flargest_free->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(flargest_free,startx_col2+lw+hor_dist,starty+2*(fh+ver_dist));
                flargest_free->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
                flargest_free->setFrameStyle(QFrame::Box|QFrame::Plain);

                // Space free
                QLabel * lspace_free = new QLabel("Space free (MB):",SpaceTab->viewport());
                lspace_free->resize(lw,lh);                
		lspace_free->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(lspace_free,startx_col2,starty+3*(lh+ver_dist));
                fspace_free = new QLabel("",SpaceTab->viewport());
                fspace_free->resize(fw,fh);
		fspace_free->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(fspace_free,startx_col2+lw+hor_dist,starty+3*(fh+ver_dist));
                fspace_free->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
                fspace_free->setFrameStyle(QFrame::Box|QFrame::Plain);

		// Tablespace size
                QLabel * lsize = new QLabel("Tablespace size (MB):",SpaceTab->viewport());
                lsize->resize(lw,lh);                
		lsize->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(lsize,150,starty+3*(lh+ver_dist)+80);
                fsize = new QLabel("",SpaceTab->viewport());
                fsize->resize(fw,fh);
		fsize->setBackgroundMode(PaletteBase);
                SpaceTab->addChild(fsize,150+lw+hor_dist,starty+3*(fh+ver_dist)+80);
                fsize->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
                fsize->setFrameStyle(QFrame::Box|QFrame::Plain);

	//SpaceTab->resizeContents(530,289);

	addTab(SpaceTab,"Space");
}

void w_dba_tablespaces_info::fillSpaceTab(const QString & tabname)
{
	if (tabname=="Space")
	{
           if (!gSQLCache)
           {
                QMessageBox::critical( 0, "SQL cache error", QString("The SQL cache is not set up"),QMessageBox::Ok | QMessageBox::Default,0,0);
                return;
           }
           if (!gConnObj)
           {
                QMessageBox::critical( 0, "Database error", "You are not connected to a database!",QMessageBox::Ok | QMessageBox::Default,0,0);
                return;
           }
	   qApp->setOverrideCursor(waitCursor);
	   try
	   {
		// Prepare the Oracle parameter
		qt_OraParm op(getWinParm()->ParmCount());
		op.addParm(getWinParm()->getParm(0));

		// Execute sql from the cache file
		gConnObj->open(gSQLCache->getSQL("extents_alloc_per_tbs"),op);
		gConnObj->fetch();

        	fextents_alloc->setText(gConnObj->getCol(0));
        	fsmallest_alloc->setText(gConnObj->getCol(1));
        	flargest_alloc->setText(gConnObj->getCol(2));
        	fspace_alloc->setText(gConnObj->getCol(3));

		gConnObj->close();


                // Execute sql from the cache file
                gConnObj->open(gSQLCache->getSQL("extents_free_per_tbs"),op);
                gConnObj->fetch();

                fextents_free->setText(gConnObj->getCol(0));
                fsmallest_free->setText(gConnObj->getCol(1));
                flargest_free->setText(gConnObj->getCol(2));
                fspace_free->setText(gConnObj->getCol(3));

                gConnObj->close();

                gConnObj->open(gSQLCache->getSQL("tbs_size"),op);
                gConnObj->fetch();

                fsize->setText(gConnObj->getCol(0));

                gConnObj->close();

	   }
	   catch (oerr & e)
	   {
	   	qApp->restoreOverrideCursor();
		QMessageBox::critical( 0, "Database error", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
		return;
	   }
	   qApp->restoreOverrideCursor();
	}
}

void w_dba_tablespaces_info::SpaceTabShown()
{
    QPopupMenu * action_menu = getActionMenu();
    action_menu->setItemEnabled(action_menu->idAt(6),false);
    action_menu->setItemEnabled(action_menu->idAt(7),false);
    action_menu->setItemEnabled(action_menu->idAt(8),false);
    action_menu->setItemEnabled(action_menu->idAt(9),false);
    action_menu->setItemEnabled(action_menu->idAt(10),false);
 
    QObjectList *list = ((QObject *)getActionToolBar())->queryList("QToolButton");
    QObjectListIt it( *list );          // iterate over the buttons
    QObject * obj;
    while ( (obj=it.current()) != 0 )
    { // for each found object...
        //if (((QString)((QToolButton *)obj)->name())=="Save as"
        if (((QString)((QToolButton *)obj)->name())=="Sort"
        ||  ((QString)((QToolButton *)obj)->name())=="Filter"
        ||  ((QString)((QToolButton *)obj)->name())=="SQL")
        {
            ((QToolButton *)obj)->setEnabled(false);
        }
        ++it;
    }
}

void w_dba_tablespaces_info::SpaceTabHidden()
{
    QPopupMenu * action_menu = getActionMenu();
    action_menu->setItemEnabled(action_menu->idAt(6),true);
    action_menu->setItemEnabled(action_menu->idAt(7),true);
    action_menu->setItemEnabled(action_menu->idAt(8),true);
    action_menu->setItemEnabled(action_menu->idAt(9),true);
    action_menu->setItemEnabled(action_menu->idAt(10),true);
 
    QObjectList *list = ((QObject *)getActionToolBar())->queryList("QToolButton");
    QObjectListIt it( *list );          // iterate over the buttons
    QObject * obj;
    while ( (obj=it.current()) != 0 )
    { // for each found object...
        //if (((QString)((QToolButton *)obj)->name())=="Save as"
        if (((QString)((QToolButton *)obj)->name())=="Sort"
        ||  ((QString)((QToolButton *)obj)->name())=="Filter"
        ||  ((QString)((QToolButton *)obj)->name())=="SQL")
        {
            ((QToolButton *)obj)->setEnabled(true);
        }
        ++it;
    }
}

void w_dba_tablespaces_info::retrieve()
{
    QString tabname=((TabDataDisplay *)getDD())->getCurrentTab();
    if (tabname!="Space")
    {
        DBSheet::retrieve();
    }
    else
    {
        fillSpaceTab(tabname);
    }
}

void w_dba_tablespaces_info::Print()
{
    QString tabname=((TabDataDisplay *)getDD())->getCurrentTab();
    if (tabname!="Space")
    {
        DBSheet::Print();
    }
    else
    {
	SpaceTab->printData();        
    }
}

void w_dba_tablespaces_info::SaveAs()
{
    QString tabname=((TabDataDisplay *)getDD())->getCurrentTab();
    if (tabname!="Space")
    {
        DBSheet::SaveAs();
    }
    else
    {
        SpaceTab->saveData();
    }
}
