/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <iostream.h>
#include <qapplication.h>
#include <qlabel.h>
#include <stdlib.h>
#include "row.h"
#include "config.h"
#include "dbtab.h"
#include "dbform.h"
#include "griddatadisplay.h"
#include "formdatadisplay.h"
#include "w_dba_users.h"
#include "w_dba_users_info.h"
#include "gensql.h"
#include "scriptdlg.h"
#include <qfile.h>
#include "editor.h"
#include <qtextstream.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>

w_dba_users::w_dba_users()
	: DBList("dba_users",0,"DBA_USERS")
{
        addGenerateSQLToActionMenu();
	show();
}

QPopupMenu * w_dba_users::addGenerateSQLToActionMenu()
{
        QPopupMenu * m= DBList::addGenerateSQLToActionMenu();
        m->insertItem("Create...",this,SLOT(generate_user_create()));
        m->insertItem("Drop...",this,SLOT(generate_user_drop()));
        m->insertItem("Become user...",this,SLOT(generate_user_become()));
        m->insertItem("Lock...",this,SLOT(generate_user_lock()));
        m->insertItem("Unlock...",this,SLOT(generate_user_unlock()));
	m->insertSeparator();
        m->insertItem("All privileges...",this,SLOT(generate_all_privs()));
        m->insertItem("Table privileges...",this,SLOT(generate_tab_privs()));
        m->insertItem("Column privileges...",this,SLOT(generate_col_privs()));
        m->insertItem("System privileges...",this,SLOT(generate_sys_privs()));
        m->insertItem("Role privileges...",this,SLOT(generate_role_privs()));
	m->insertSeparator();
        m->insertItem("All grants...",this,SLOT(generate_all_grants()));
        m->insertItem("Table grants...",this,SLOT(generate_tab_grants()));
        m->insertItem("Column grants...",this,SLOT(generate_col_grants()));
        return m;
}

void w_dba_users::callInfo(row * clickedRow)
{
	w_dba_users_info *wui = new w_dba_users_info(clickedRow->getCellbyName("username"));
}

void w_dba_users::generate_tab_privs()
{
	generate_sql("tab_privs.sql","tab_privs");
}

void w_dba_users::generate_role_privs()
{
	generate_sql("role_privs.sql","role_privs");
}

void w_dba_users::generate_sys_privs()
{
	generate_sql("sys_privs.sql","sys_privs");
}

void w_dba_users::generate_col_privs()
{
	generate_sql("col_privs.sql","col_privs");
}

void w_dba_users::generate_all_privs()
{
	generate_sql("all_privs.sql","all_privs");
}

void w_dba_users::generate_tab_grants()
{
	generate_sql("tab_grants.sql","tab_grants");
}

void w_dba_users::generate_col_grants()
{
	generate_sql("col_grants.sql","col_grants");
}

void w_dba_users::generate_all_grants()
{
	generate_sql("all_grants.sql","all_grants");
}

void w_dba_users::generate_user_create()
{
	generate_sql("user_create.sql","user_create");
}

void w_dba_users::generate_user_lock()
{
	generate_sql("user_lock.sql","user_lock");
}

void w_dba_users::generate_user_unlock()
{
	generate_sql("user_unlock.sql","user_unlock");
}

void w_dba_users::generate_user_drop()
{
	generate_sql("user_drop.sql","user_drop");
}

void w_dba_users::generate_user_become()
{
	generate_sql("user_become.sql","user_become");
}

void w_dba_users::generate_sql(const QString& fn, const QString& what)
{
        QString object_name,filename,owner;
        bool    allobj, show_script, echo_on, spool;

        gConfigFile->setGroup("PATH");
        QString path = gConfigFile->readEntry("sql_generation",".");
 
        ScriptDlg *sd = new ScriptDlg(path+"/"+fn,this);
        if (sd->exec())
        {
		qApp->setOverrideCursor(waitCursor);
                filename=sd->getFilename();
                allobj=sd->getAllObj();
                show_script=sd->getShow();
		spool=sd->getSpool();
		echo_on=sd->getEchoOn();
                delete sd;
		sd=0;

                QFile f(filename);
                if (f.open(IO_WriteOnly))
                {
                        QTextStream t(&f);
			if (spool) t << "spool " << filename << ".log" << endl;
			if (echo_on) t << "set echo on" << endl;
                        if (!allobj) // generate script for selected object
                        {
                                object_name=((row *)(((GridDataDisplay *)getDD())->currentItem()))->getCellbyName("username");
				int retval;
				if (what=="tab_privs")
					retval=GenSQL::UserTabPrivs(t,object_name);
				else if (what=="user_create")
					retval=GenSQL::UserCreate(t,object_name);
				else if (what=="user_drop")
					retval=GenSQL::UserDrop(t,object_name);
				else if (what=="user_become")
					retval=GenSQL::UserBecome(t,object_name);
				else if (what=="role_privs")
					retval=GenSQL::UserRolePrivs(t,object_name);
				else if (what=="sys_privs")
					retval=GenSQL::UserSysPrivs(t,object_name);
				else if (what=="col_privs")
					retval=GenSQL::UserColPrivs(t,object_name);
				else if (what=="all_privs")
					retval=GenSQL::UserPrivs(t,object_name);
				else if (what=="tab_grants")
					retval=GenSQL::UserTabGrants(t,object_name);
				else if (what=="col_grants")
					retval=GenSQL::UserColGrants(t,object_name);
				else if (what=="all_grants")
					retval=GenSQL::UserGrants(t,object_name);
				else if (what=="user_lock")
					retval=GenSQL::UserLock(t,object_name);
				else if (what=="user_unlock")
					retval=GenSQL::UserUnlock(t,object_name);
                                if (retval==-1)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "File write error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                                else if (retval==-2)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "Database error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                                else if (retval==-3)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "SQL cache error. SQL could not be found!\nProcessing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                        }
                        else // generate script for all objects
                        {
                                QListViewItemIterator it((GridDataDisplay *)getDD());
                                // loop over all rows
                                for (;it.current(); ++it)
                                {
                                        object_name=((row *)it.current())->getCellbyName("username");
					int retval;
					if (what=="tab_privs")
						retval=GenSQL::UserTabPrivs(t,object_name);
					else if (what=="role_privs")
						retval=GenSQL::UserRolePrivs(t,object_name);
					else if (what=="user_create")
						retval=GenSQL::UserCreate(t,object_name);
					else if (what=="user_drop")
						retval=GenSQL::UserDrop(t,object_name);
					else if (what=="user_become")
						retval=GenSQL::UserBecome(t,object_name);
					else if (what=="sys_privs")
						retval=GenSQL::UserSysPrivs(t,object_name);
					else if (what=="col_privs")
						retval=GenSQL::UserColPrivs(t,object_name);
					else if (what=="all_privs")
						retval=GenSQL::UserPrivs(t,object_name);
					else if (what=="tab_grants")
						retval=GenSQL::UserTabGrants(t,object_name);
					else if (what=="col_grants")
						retval=GenSQL::UserColGrants(t,object_name);
					else if (what=="all_grants")
						retval=GenSQL::UserGrants(t,object_name);
					else if (what=="user_lock")
						retval=GenSQL::UserLock(t,object_name);
					else if (what=="user_unlock")
						retval=GenSQL::UserUnlock(t,object_name);
                                        if (retval==-1)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "File write error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                        else if (retval==-2)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "Database error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                	else if (retval==-3)
                                	{
                                        	QMessageBox::critical( 0, "SQL generation error", "SQL cache error. SQL could not be found!\nProcessing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        	f.close();
                                        	return;
                                	}
                                }
                        }
			if (echo_on) t << "set echo off" << endl;
			if (spool) t << "spool off" << endl;
                        f.close();
                }

                if (show_script)
                {
                        Editor * ed = new Editor(filename);
                        ed->show();
                }
		qApp->restoreOverrideCursor();
        }
        else
        {
                delete sd;
		sd=0;
        }
}
