/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "w_locktree.h"
#include <qmessagebox.h>
#include <qtextstream.h>
#include "gensql.h"
#include "scriptdlg.h"
#include "editor.h"
#include "config.h"
#include "row.h"
#include <qfile.h>
#include <iostream.h>
#include "sqlcache.h"

w_locktree::w_locktree()
	: DBList("lock_holders",0,"Locktree")
{
        addGenerateSQLToActionMenu();
	getLockWaiters();
	show();
}

void w_locktree::getLockWaiters()
{
	GridDataDisplay *tree=(GridDataDisplay *)getDD();	
	tree->setRootIsDecorated(true);
	// Get the waiting sessions
        QString sql;
	if (gSQLCache) sql=gSQLCache->getSQL("lock_waiters");
        else
        {
                QMessageBox::critical( 0, "SQL cache error", QString("The SQL cache is not set up"),QMessageBox::Ok | QMessageBox::Default,0,0);
                return;
        }
        if (sql=="")
	{
                QMessageBox::critical( 0, "SQL cache error", QString("The following SQL could not be found: lock_waiters"),QMessageBox::Ok | QMessageBox::Default,0,0);
		return;
	}
        GridDataDisplay* waiters = new GridDataDisplay(gConnObj,sql);
        if (waiters->retrieve()==-1) 
	{
		// BETTER ERROR HANDLING HERE
		return;
	}
	row *current_waiter;
	row *current_holder=(row *)tree->firstChild();
	for (;current_holder; current_holder=(row *)current_holder->nextSibling())
        {
		QString holder_id1=current_holder->getCellbyName("id1");
		QString holder_id2=current_holder->getCellbyName("id2");
		QListViewItemIterator waiters_it(waiters);
		current_waiter=(row *)waiters->firstChild();
		for (;current_waiter; )
		{
			QString waiter_id1=current_waiter->getCellbyName("id1");
			QString waiter_id2=((row *)waiters_it.current())->getCellbyName("id2");
			if (waiter_id1==holder_id1 && waiter_id2==holder_id2)
			{
				waiters->takeItem(current_waiter);
				current_holder->insertItem(current_waiter);
				current_waiter=(row *)waiters->firstChild();
				tree->setOpen(current_holder,true);
			}
			else
			{
				current_waiter=(row *)current_waiter->nextSibling();
			}
		}
	}
        delete waiters;
        waiters=0;
}

void w_locktree::retrieve()
{
	DBSheet::retrieve();
	getLockWaiters();
}

//void w_locktree::callInfo(row * clickedRow)
//{
//	w_locktree_info *wui = new w_locktree_info(clickedRow->getCellbyName("sid"),clickedRow->getCellbyName("username"));
//}

QPopupMenu * w_locktree::addGenerateSQLToActionMenu()
{
        QPopupMenu * m= DBList::addGenerateSQLToActionMenu();
        m->insertItem("Kill...",this,SLOT(generate_session_kill()));
        return m;
}

void w_locktree::generate_session_kill()
{
        generate_sql("session_kill.sql","session_kill");
}

void w_locktree::generate_sql(const QString& fn, const QString& what)
{
        QString sid,filename,serial;
        bool    allobj, show_script, echo_on, spool;

        gConfigFile->setGroup("PATH");
        QString path = gConfigFile->readEntry("sql_generation",".");

        ScriptDlg *sd = new ScriptDlg(path+"/"+fn,this);
        if (sd->exec())
        {
		qApp->setOverrideCursor(waitCursor);
                filename=sd->getFilename();
                allobj=sd->getAllObj();
                show_script=sd->getShow();
                spool=sd->getSpool();
                echo_on=sd->getEchoOn();
                delete sd;
		sd=0;

                QFile f(filename);
                if (f.open(IO_WriteOnly))
                {
                        QTextStream t(&f);
                        if (spool) t << "spool " << filename << ".log" << endl;
                        if (echo_on) t << "set echo on" << endl;
                        if (!allobj) // generate script for selected object
                        {
                                sid=((row *)(((GridDataDisplay *)getDD())->currentItem()))->getCellbyName("sid");
                                serial=((row *)(((GridDataDisplay *)getDD())->currentItem()))->getCellbyName("serial#");
                                int retval;
                                if (what=="session_kill")
                                        retval=GenSQL::SessionKill(t,sid,serial);
                                else if (what=="session_trace")
                                        retval=GenSQL::SessionTrace(t,sid,serial);
                                if (retval==-1)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "File write error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                                else if (retval==-2)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "Database error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                                else if (retval==-3)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "SQL cache error. SQL could not be found!\nProcessing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                        }
                        else // generate script for all objects
                        {
                                QListViewItemIterator it((GridDataDisplay *)getDD());
                                // loop over all rows
                                for (;it.current(); ++it)
                                {
                                        sid=((row *)it.current())->getCellbyName("sid");
                                        serial=((row *)it.current())->getCellbyName("serial#");
                                        int retval;
                                        if (what=="session_kill")
                                                retval=GenSQL::SessionKill(t,sid,serial);
                                	else if (what=="session_trace")
                                        	retval=GenSQL::SessionTrace(t,sid,serial);
                                        if (retval==-1)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "File write error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                        else if (retval==-2)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "Database error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                        else if (retval==-3)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "SQL cache error. SQL could not be found!\nProcessing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                }
                        }
                        if (echo_on) t << "set echo off" << endl;
                        if (spool) t << "spool off" << endl;
                        f.close();
                }

                if (show_script)
                {
                        Editor * ed = new Editor(filename);
                        ed->show();
                }
		qApp->restoreOverrideCursor();
        }
        else
        {
                delete sd;
		sd=0;
        }
}

